/*
 * Arduino sketch to communicate with HP 85064-60002 eCal unit (also works with other 85062 and 4 series eCals)
 * 
 * Wayne Knowles  ZL2BKC   27-Sep-2021
 * w.knowles@xtra.co.nz
 * 
 * Interface Details:
 * 
 * **Note** 5V Interface levels required - check compatability with Arduino
 * 
 * Arduino Pin  DB25 Pin  Port  Dir   Description
 * --------------------------------------------------------------------
 *     D0                             Reserved TX Out
 *     D1                             Reserved RX In
 *     D2       13        PD2   I/O   ECal Data bus bit 0
 *     D3       25        PD3   I/O   ECal Data bus bit 1
 *     D4       12        PD4   I/O   ECal Data bus bit 2
 *     D5       24        PD5   I/O   ECal Data bus bit 3
 *     D6       11        PD6   I/O   ECal Data bus bit 4
 *     D7       23        PD7   I/O   ECal Data bus bit 5
 *     D8       10        PB0   I/O   ECal Data bus bit 6
 *     D9       22        PB1   I/O   ECal Data bus bit 7
 *     D10       9        PB2   Out   Mux A0
 *     D11      21        PB3   Out   Mux A1
 *     D12       8        PB4   Out   Mux A2
 *     D13      20        PB5   Out   ClkP [Data latched on rising edge]
 *     A0        7        PA0   Out   OE to EEPROM
 *     A1       19        PA1   Out   WR to EEPROM
 *     A2        6        PA2   Out   MR Master Reset all latches
 * 
 *     GND       2                    Digital Ground
 *     GND      15                    Digital Ground
 *     +5V       3                    VCC Digital    (< 50mA - OK to power from Arduino)           
 *     
 *     +5V      16                    Analog 5v
 *     +24V      1                    Analog +20V (20-24V @ 750mA Startup)
 *     GND      14                    Analog Ground
 */

uint16_t output_bits;    //  Control lines that drive transmission line gates

void reset() {
  digitalWrite(A2, 0);  // Assert L.MR to reset interface
  digitalWrite(A2, 1);  // L.MR  
}


void set_data(uint8_t data) {
  DDRD |= 0xFC;         //  Top 6 bits are output (lower 2 are RS232 noise) 
  DDRB |= 0x03;         //  Remaining bits at bottom of PORT B (D9 D10) 
  PORTD = data << 2;    // 
  PORTB = (PORTB & 0xFC) | (data >> 6);
}

void set_mux(uint8_t addr) {
  DDRB |= 0b00011100;         //  Set Address lines as output
  PORTB = (PORTB & ~0b00011100) | (addr&0x7) << 2 ;
}

void strobe() {
    digitalWrite(13, 0);  //  Activate strobe
    digitalWrite(13, 1);  //  Data clocked by rising edge
}


void write_reg(uint8_t reg, uint8_t data) {
  set_data(data);
  set_mux(reg & 0x07);
  strobe();
}

//  Send sequence to read 1 byte from 
uint8_t read_eeprom(uint32_t addr) {
   uint8_t  data;
   
   write_reg(2, addr & 0xFF);
   write_reg(3, (addr>>8) & 0xFF);
   write_reg(4, (addr>>16) & 0xFF);   

   // Set DDR for reading
   PORTD &= ~0xFC;        //  Disable any pullup
   DDRD &= ~0xFC;         //  Top 6 bits are output (lower 2 are RS232 noise) 
   PORTB &= ~0x03;        //  Disable any pullup
   DDRB &= ~0x03;         //  Remaining bits at bottom of PORT B (D9 D10) 
  
   set_mux(7);     // Address EEPROM
   digitalWrite(A0, 0);  // L.OE active to read EEPROM
   digitalWrite(13, 0);  //  Activate strobe which also activates CE for EEPROM
   data =  (PIND >> 2) | (PINB & 0x03) << 6;
   digitalWrite(13, 1);  //  Data clocked by rising edge

   digitalWrite(A0, 1);  // Disable L.OE 

   //  Leave bus in read mode - will reset as part of next write_reg
   return (data); 
}



void readline(char *buf, byte len)
{
  char *p = buf;
  char c;
  do
  {
    while (Serial.available() == 0) ;
    c = Serial.read();
    Serial.write(c);
    if (len)  { *p++ = c; len--; }
  }
  while (c != '\n' && c != '\r'); 
  *(--p) = (char) 0;
  Serial.write('\n');
}

void update_eCal() {
    write_reg(1, (output_bits>>8) & 0xFF); // Write MSB  
    write_reg(0, output_bits & 0xFF);      // Write LSB
}

// the setup routine runs once when you press reset:
void setup() {
  int pin;
  
  // initialize serial communication at 9600 bits per second:
  Serial.begin(115200);
  for (pin = 2; pin <= 13; pin++)
    pinMode(pin, OUTPUT);

  for (pin = A0; pin <= A5; pin++)
    pinMode(pin, OUTPUT);

  digitalWrite(A0, 1);  // L.OE
  digitalWrite(A1, 1);  // L.WR
  digitalWrite(A2, 1);  // L.MR
  reset();

  output_bits = 0;
  update_eCal();    // Update the control gates in eCal

  Serial.println("\nHP 85062/85064 eCal manual control\n");
  cmd_info();
  Serial.print("> ");
}


// the loop routine runs over and over again forever:
void loop() {
  char  buf[30];
  int  n;

  readline(buf, 30);
  Serial.print("\n");
  exec_command(buf);
  Serial.print("> ");

}
