
#define MAXARGS   5


extern uint16_t output_bits;

// Walk across all 16 FET control pins - terminate when 
void cmd_walk() {
  uint16_t  bits;
  uint8_t   i, ndot = 0;
  
    Serial.println("Walking bits 1 through 16.  Press any key to terminate");

    while (!Serial.available()) {
        for (i=0; i<=15; i++) {
          bits = 1U <<i;
      
          write_reg(1, (bits>>8) & 0xFF); // Write MSB  
          write_reg(0, bits & 0xFF);      // Write LSB
            
          delay(1);    
        }
        Serial.write('.');
        if (++ndot >= 80) {
            ndot = 0;
            Serial.write('\n');
        }
    }
    while (Serial.available()) {
      Serial.read();    // Swallow the byte just typed
      delay(1);
    }
    Serial.println("*Terminated*");
}

uint16_t read_eeprom_short(uint32_t addr) {
  byte b1, b2;
  b1 = read_eeprom(addr++);
  b2 = read_eeprom(addr);
  return b1 + (b2 << 8);
}

void eeprom_print(uint32_t addr) {
    byte c;
    
    c = read_eeprom(addr++);
    while (c && c != 0xFF) {
        Serial.write(c);
        c = read_eeprom(addr++);
    }
    Serial.println();
}

void cmd_info() {
  uint16_t npoints = read_eeprom_short(0x00DC);
  uint16_t warmup = read_eeprom_short(0x00DE);

  if (npoints == 0xFFFF || warmup == 0xFFFF || npoints == 0) {
      Serial.println("Module not responding");
      return;
  }
  Serial.print("Model Number            : ");   eeprom_print(0x00F0);
  Serial.print("Connector Type          : ");   eeprom_print(0x0070);
  Serial.print("Last Certification      : ");   eeprom_print(0x0084);
  Serial.print("Module number of points : ");   Serial.print(npoints);  Serial.println("");
  Serial.print("Recommended warmup time : ");   Serial.print(warmup);  Serial.println("s");
}

//
static void  DumpHex(uint32_t start_addr, uint32_t end_addr)
{
int       ii;
int       c;
int       lineCount;
char      textString[16];
char      buf[24];
uint32_t  addr_ptr;

  lineCount     = 0;
  addr_ptr  = start_addr;
  while (addr_ptr < end_addr)
  {
    sprintf(textString, "%06lX: ", addr_ptr);
    Serial.print(textString);
    
    buf[0]    = 0;
    for (ii=0; ii<16; ii++)
    {
      c  = read_eeprom(addr_ptr);

      sprintf(textString, "%02X ", c);
      Serial.print(textString);
      if ((c >= 0x20) && (c < 0x7f))
      {
        buf[ii % 16]  = c;
      }
      else
      {
        buf[ii % 16]  = '.';
      }
      addr_ptr++;
    }
    buf[16] = 0;
    Serial.print(" | ");
    Serial.print(buf);
    Serial.println(" | ");
    lineCount++;
    if (Serial.available()) {
       Serial.println("**Interrupt**");
       break;
    }
  }
  while (Serial.available()) {    // Flush RX Input
      (void) Serial.read();
  }
}

void cmd_dump(uint32_t addr1, uint32_t addr2) {
    DumpHex(addr1, addr2);
}

//  Set of clear a specific bit.    op = 0 to reset and 1 to set
void cmd_set_clr(char *buf, byte op) {
    byte start_b, end_b, i;
    char *p;

    output_bits = 0;
    
    p = buf;
    while (*p) {
        if (*p == 'a') {  // all command
           output_bits = op?0xFFFF:0x0000; break;
        } else if (*p == 'n') {
           output_bits = op?0x0000:0xFFFF;  break;
        } else {
            start_b = strtoul(p, &p, 10);
            if (*p == '-' || *p == ':') { // Range specified 
                end_b = strtoul(++p, &p, 10);
                for (i = start_b; i <= end_b; i++)
                    output_bits |= 1U << (i-1);
            } else {
                if (op)
                    output_bits |= 1U << (start_b-1);
                else
                    output_bits &= ~(1U << (start_b-1));
            }
            if (*p == ',') { 
                p++;  // Get ready for the next
                continue;
            }
        } 
        if (*p) {
            Serial.println("Input Error");
            return;
        }
    }
 
    update_eCal();
    Serial.println("        :                   1 1 1 1 1 1 1 :");
    Serial.println("  Gate  : 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 :");
    Serial.print  ("Status  :");
    for (i=0; i<16; i++) {
       Serial.print( output_bits & (1U<<i)?" X":" -");
    }
    Serial.println(" :");
}

void cmd_help() {

    Serial.println("dump [start] [end]    Dump EEPROM data"); 
    Serial.println("info         Display ECal revision details");
    Serial.println("set n        Enable switch n");
    Serial.println("set n,m,...  Multiple values seperated by comma");
    Serial.println("set n1-n2    Range with dash");
    Serial.println("set all      Enable all switches");
    Serial.println("set none     Disable all switches"); 
    Serial.println("walk         Walk switch bits 1 through 16");        
}

void exec_command(char *buf)
{
  int16_t   intarg;

  char *argv[MAXARGS];
  uint8_t argc, v;
  
  argc = 0;
  v = 0;
  while (*buf && *buf == ' ') *buf++=(char)0; // skip spaces
  for (v=0; *buf && v<MAXARGS; v++) {
    // replace spaces with null terminator to help terminate each word
    argv[v] = buf;
    argc++;
    while (*buf && *buf != ' ') buf++;  // digest command
    while (*buf && *buf == ' ') *buf++=(char)0; // skip spaces
  }
  for (; v<MAXARGS; v++)
    argv[v] = buf;    //  Point to NULL terminator

  if (argc == 0)
    return;  // No command

  intarg = atol(argv[1]);

  if (strcasecmp_P(argv[0], PSTR("walk"))==0) {
      cmd_walk();
      return;
  }
  if (strcasecmp_P(argv[0], PSTR("dump"))==0) {
      uint32_t  addr1, addr2;
      addr1 = 0;
      addr2 = 0x100;

      if (argc >= 2) { addr1 = strtoul(argv[1], NULL, 16); addr2 = addr1 + 0x10; }
      if (argc >= 3) { addr2 = strtoul(argv[2], NULL, 16); }     
      cmd_dump(addr1, addr2);
      return;
  }
  if (strcasecmp_P(argv[0], PSTR("set"))==0) {  
      cmd_set_clr(argv[1], 1);
      return;
  }
  if (strcasecmp_P(argv[0], PSTR("clr"))==0) {  
      cmd_set_clr(argv[1], 0);
      return;
  }
  if (strcasecmp_P(argv[0], PSTR("info"))==0) {  
      cmd_info();
      return;
  }
  if (strcasecmp_P(argv[0], PSTR("help"))==0) {  
      cmd_help();
      return;
  }
  Serial.println("Unknown Command.  Type 'help' for list of commands");
}
