#! /usr/bin/python

import sys,os 
import struct
import string
import cmath, math, numpy

PY3K=True

def dehex(hextext):
  """
  Convert from hex string to binary data stripping
  whitespaces from `hextext` if necessary.
  """
  if PY3K:
    return bytes.fromhex(hextext)
  else:
    hextext = "".join(hextext.split())
    return hextext.decode('hex')

	
def restore(dump):
  '''
  Restore binary data from a hex dump.
    [x] dump argument as a string
    [ ] dump argument as a line iterator

  Supported formats:
    [x] hexdump.hexdump
    [x] Scapy
    [x] Far Manager
  '''
  minhexwidth = 2*16    # minimal width of the hex part - 00000... style
  bytehexwidth = 3*16-1 # min width for a bytewise dump - 00 00 ... style

  result = bytes() if PY3K else ''
  if type(dump) != str:
    raise TypeError('Invalid data for restore')

  text = dump.strip()  # ignore surrounding empty lines
  for line in text.split('\n'):
    # strip address part
    addrend = line.find(':')
    if 0 < addrend < minhexwidth:  # : is not in ascii part
      line = line[addrend+1:]
    line = line.lstrip()
    # check dump type
    if line[2] == ' ':  # 00 00 00 ...  type of dump
      # check separator
      sepstart = (2+1)*7+2  # ('00'+' ')*7+'00'
      sep = line[sepstart:sepstart+3]
      if sep[:2] == '  ' and sep[2:] != ' ':  # ...00 00  00 00...
        hexdata = line[:bytehexwidth+1]
      elif sep[2:] == ' ':  # ...00 00 | 00 00...  - Far Manager
        hexdata = line[:sepstart] + line[sepstart+3:bytehexwidth+2]
      else:                 # ...00 00 00 00... - Scapy, no separator
        hexdata = line[:bytehexwidth]
      line = hexdata
    result += dehex(line)
    return result

def read_string(data, pos, max=256):
	p = pos
	while (data[p] != 0 and p-pos < max):
		p = p + 1
	return data[pos:p].decode('UTF-8')

def read_uint16(data, pos):
	return int.from_bytes(data[pos:pos+2], byteorder='little')

def read_double(data, pos):
	return float.from_bytes(data[pos:pos+8], byteorder='little')

class Touchstone:

	def __init__(self, freq, data, nsets=1, Z0=50):
		self._freq = freq
		self._sdata = data
		self._nsets = nsets
		self._points = len(freq)
		self._Z0 = Z0
		
	def write_file(self, filename):
		outfile = open(filename, "w")
		outfile.write("# MHZ S DB R %d\n" % self._Z0)
		outfile.write("!\n")
		
		for i in range(0, self._nsets):
			out = "%14f" % (self._freq[i]/1e6)
			for sweep in self._sparams:
				value = self._sdata[sweep][i]
				out = out + " %12.7f %12.7f"% (20*log10(abs(value)),
											angle(value, deg=True))
			outfile.write(out)
			outfile.write("\n")
		outfile.close()
	

class ECal:
	def __init__(self):
		pass
		
	def import_dump(self, file):
		"""Import EEPROM dump output taken from Arduino sketch"""
		
		eedata = bytes()
		file = open(file, 'r')
		for line in file.readlines():
			line = line.strip()
			if len(line) >= 50 and line[6] == ':' and line[57] == '|':
				a = dehex(line[0:6])
				addr = int.from_bytes(a, byteorder='big')
				d = dehex(line[8:55])
				assert(addr ==len(eedata))
				eedata = eedata + d
		self.eedata = eedata

	def extract(self):
		
		self.start, self.stop, none, self.step = struct.unpack_from("<2did", self.eedata, 0x000CC)
		self.npoints, freq_table_loc =  struct.unpack_from("<HI", self.eedata, 0x0190)
		
		self.freq_list = struct.unpack_from("<"+str(self.npoints)+"d",  self.eedata, freq_table_loc)

		self.model_number = read_string(self.eedata, 0x000F0)
		self.serial_number = read_string(self.eedata, 0x00064)
		self.connector_type = read_string(self.eedata, 0x00070)
		self.cert_date = read_string(self.eedata, 0x00084)
		self.warmup_time = read_uint16(self.eedata, 0x000DE)
	
	def pod_info(self, prefix = "", sep = " ", endl = "\n"):
		info = [
		("Model Number            :", self.model_number),
		("Serial Number           :", self.serial_number),
		("Connector Type          :", self.connector_type),
		("Last Certification      :", self.cert_date),
		("Start Frequency         :", "%.lf MHz" % (self.start / 1.0e6)),
		("End Frequency           :", "%.lf MHz" % (self.stop / 1.0e6)),  
		("Step                    :", "%.lf MHz" % (self.step / 1.0e6)), 
		("Number of points        :", "%d" % self.npoints), 
		("Recommended warmup time :", "%u seconds" % self.warmup_time)	
		]

		out = ""
		for l in info:
			out = out + prefix + sep.join(l) + endl
		return out

	def dump_dataset(self, file, offset, nsweeps):
		dataset = struct.unpack_from("<"+str(self.npoints*2*nsweeps)+"f", self.eedata, offset)
		Z0 = 50
		
		file.write("# MHZ S DB R %d\n"%Z0)
		for i in range(0, self.npoints):
			out = "%14f" % (self.freq_list[i]/1e6)
			for j in range(0, nsweeps):
				real, imag = dataset[(j*self.npoints+i)*2], dataset[(j*self.npoints+i)*2+1]
				value = complex(real, imag)
				out = out + " %12.7f %12.7f"% (20*math.log10(abs(value)), numpy.angle(value, deg=True))
			file.write(out + "\n")

	def dump_tables(self):
		set = 1
		for table in range(0x000196, 0x0001CE, 0x000E):		# Dump 4 tables
		
			num_cals, set_size, num_sets, switch_start, dataset_start =  \
				struct.unpack_from("<HHHII", self.eedata, table)
			
			num_sweeps = int(num_sets / num_cals)	# Number of sweeps per dataset
		
			
			switch_table = struct.unpack_from("<"+str(num_cals)+"H",  self.eedata, switch_start)
			print()
			print("***********************************************")
			print("**  Switch Data: ", " ".join([f"0x{i:04X}" for i in switch_table]) ) 
			print("***********************************************")
			
			offset = dataset_start
			for switch in switch_table:
				if num_sweeps > 1:	extention = "s2p"
				else:				extention = "s1p"
				filename = "ECal_HP_%s_SN%s_set%d_sw#%04X.%s" % \
					(self.model_number, self.serial_number, set, switch, extention)
				print ("Writing file", filename)
				file = open(filename, "w")

				file.write("! ListType=Lin\n")
				file.write("!!ECal Dump set=%s switches=#%04X\n"%(set, switch))
				file.write("!\n")
				file.write("! ECal Config Switch:  0x%04X\n"%switch)
				file.write("! EEPROM Table Addr:   0x%04X\n"%offset)
				file.write("!\n")
				file.write(ecal.pod_info(prefix="! "))
				file.write("!\n")
				file.write("! Freq             DB     Phase\n")
				file.write("! ------------------------------------\n")
				self.dump_dataset(file, offset, num_sweeps)
				file.write("! ------------------------------------\n")
				file.close()
				
				offset = offset+self.npoints*8   # 8 bytes per point (2 x 32 bit float values) 
			set = set+1

if __name__ == "__main__":			
	ecal = ECal()
	ecal.import_dump(sys.argv[1])	
	ecal.extract()
	print(ecal.pod_info())

	ecal.dump_tables()
		

